@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Main module code templates
 
 This for the main program in case of executables, the module templates and
@@ -471,6 +457,13 @@ PyObject *modulecode_%(module_identifier)s(PyThreadState *tstate, PyObject *modu
     // Report to PGO about leaving the module without error.
     PGO_onModuleExit("%(module_identifier)s", false);
 
+#if defined(_NUITKA_MODULE) && %(is_top)d
+    PyObject *post_load = IMPORT_EMBEDDED_MODULE(tstate, %(module_name_cstr)s "-postLoad");
+    if (post_load == NULL) {
+        return NULL;
+    }
+#endif
+
     Py_INCREF(module_%(module_identifier)s);
     return module_%(module_identifier)s;
 %(module_exit)s
@@ -517,9 +510,10 @@ template_module_external_entry_point = r"""
 #define MOD_INIT_DECL(name) NUITKA_MODULE_INIT_FUNCTION PyInit_##name(void)
 #endif
 
+static PyObject *orig_dunder_file_value;
+
 #if PYTHON_VERSION >= 0x300
 static setattrofunc orig_PyModule_Type_tp_setattro;
-static PyObject *orig_dunder_file_value;
 
 /* This is used one time only. */
 static int Nuitka_TopLevelModule_tp_setattro(PyObject *module, PyObject *name, PyObject *value) {
@@ -566,6 +560,18 @@ static struct PyModuleDef mdef_%(module_identifier)s = {
 // Actual name might be different when loaded as a package.
 static char const *module_full_name = %(module_name_cstr)s;
 
+#if PYTHON_VERSION < 0x300
+static void onModuleFileValueRelease(void *v) {
+    if (orig_dunder_file_value != NULL) {
+        UPDATE_STRING_DICT0(
+            moduledict_%(module_identifier)s,
+            (Nuitka_StringObject *)const_str_plain___file__,
+            orig_dunder_file_value
+        );
+    }
+}
+#endif
+
 MOD_INIT_DECL(%(module_identifier)s) {
     if (_Py_PackageContext != NULL) {
         module_full_name = _Py_PackageContext;
@@ -598,9 +604,21 @@ MOD_INIT_DECL(%(module_identifier)s) {
 #if PYTHON_VERSION < 0x300
     modulecode_%(module_identifier)s(tstate, module, NULL);
 
-    // TODO: Our "__file__" value will not be respected by CPython and one
-    // way we could avoid it, is by having a capsule type, that when it gets
-    // released, we are called.
+    // Our "__file__" value will not be respected by CPython and one
+    // way we can avoid it, is by having a capsule type, that when
+    // it gets released, we are called and repair the value.
+
+    if (HAS_ERROR_OCCURRED(tstate) == false) {
+        orig_dunder_file_value = DICT_GET_ITEM_WITH_HASH_ERROR1(tstate, (PyObject *)moduledict_%(module_identifier)s, const_str_plain___file__);
+
+        PyObject *fake_file_value = PyCObject_FromVoidPtr(NULL, onModuleFileValueRelease);
+
+        UPDATE_STRING_DICT1(
+            moduledict_%(module_identifier)s,
+            (Nuitka_StringObject *)const_str_plain___file__,
+            fake_file_value
+        );
+    }
 #else
     PyObject *result = modulecode_%(module_identifier)s(tstate, module, NULL);
 
@@ -660,3 +678,18 @@ template_header_guard = """\
 from . import TemplateDebugWrapper  # isort:skip
 
 TemplateDebugWrapper.checkDebug(globals())
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
