@@ -1,20 +1,6 @@
-#     Copyright 2021, Batakrishna Sahu, mailto:<Batakrishna.Sahu@suiit.ac.in>
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Batakrishna Sahu, mailto:<Batakrishna.Sahu@suiit.ac.in> find license text at end of file
+
+
 """ Node for the calls to the 'any' built-in.
 
 """
@@ -22,22 +8,20 @@
 from nuitka.specs import BuiltinParameterSpecs
 
 from .ExpressionBases import ExpressionBuiltinSingleArgBase
+from .ExpressionShapeMixins import ExpressionBoolShapeExactMixin
 from .NodeMakingHelpers import (
     makeConstantReplacementNode,
     makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue,
     wrapExpressionWithNodeSideEffects,
 )
-from .shapes.BuiltinTypeShapes import tshape_bool
 
 
-class ExpressionBuiltinAny(ExpressionBuiltinSingleArgBase):
+class ExpressionBuiltinAny(
+    ExpressionBoolShapeExactMixin, ExpressionBuiltinSingleArgBase
+):
     """Builtin Any Node class.
 
-    Args:
-        ExpressionBase: 'any - expression'
-
-    Returns:
-        Node that represents built-in 'any' call.
+    Node that represents built-in 'any' call.
 
     """
 
@@ -49,6 +33,9 @@ class ExpressionBuiltinAny(ExpressionBuiltinSingleArgBase):
         value = self.subnode_value
         shape = value.getTypeShape()
         if shape.hasShapeSlotIter() is False:
+            # An exception is raised.
+            trace_collection.onExceptionRaiseExit(BaseException)
+
             return makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(
                 template="'%s' object is not iterable",
                 operation="any",
@@ -111,11 +98,6 @@ class ExpressionBuiltinAny(ExpressionBuiltinSingleArgBase):
 
         return self, None, None
 
-    @staticmethod
-    def getTypeShape():
-        """returns type shape of the 'any' node"""
-        return tshape_bool
-
     def mayRaiseException(self, exception_type):
         """returns boolean True if exception is raised else False"""
         value = self.subnode_value
@@ -124,3 +106,19 @@ class ExpressionBuiltinAny(ExpressionBuiltinSingleArgBase):
             return True
 
         return not value.getTypeShape().hasShapeSlotIter()
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
