@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Module/Package nodes
 
 The top of the tree. Packages are also modules. Modules are what hold a program
@@ -57,6 +43,7 @@ class PythonModuleBase(NodeBase):
 
     def __init__(self, module_name, reason, source_ref):
         assert type(module_name) is ModuleName, module_name
+        assert module_name != "", source_ref
 
         NodeBase.__init__(self, source_ref=source_ref)
 
@@ -194,7 +181,10 @@ class PythonModuleBase(NodeBase):
             current = filename
 
             levels = full_name.count(".")
-            if self.isCompiledPythonPackage():
+            if (
+                self.isCompiledPythonPackage()
+                and not self.isCompiledPythonNamespacePackage()
+            ):
                 levels += 1
 
             for _i in range(levels):
@@ -202,6 +192,13 @@ class PythonModuleBase(NodeBase):
 
                 result = os.path.join(os.path.basename(current), result)
 
+            # Avoid unnecessary unicode path values
+            if str is not bytes:
+                try:
+                    result = str(result)
+                except UnicodeDecodeError:
+                    pass
+
             return result
 
 
@@ -712,6 +709,10 @@ class CompiledPythonPackage(CompiledPythonModule):
             source_ref=source_ref,
         )
 
+    @staticmethod
+    def isCompiledPythonPackage():
+        return True
+
     def getOutputFilename(self):
         result = self.getFilename()
 
@@ -725,6 +726,14 @@ class CompiledPythonPackage(CompiledPythonModule):
         return not hasPythonFlagIsolated()
 
 
+class CompiledPythonNamespacePackage(CompiledPythonPackage):
+    kind = "COMPILED_PYTHON_NAMESPACE_PACKAGE"
+
+    @staticmethod
+    def isCompiledPythonNamespacePackage():
+        return True
+
+
 def makeUncompiledPythonModule(
     module_name, reason, filename, bytecode, is_package, technical
 ):
@@ -939,7 +948,7 @@ class PythonMainModule(CompiledPythonModule):
             )
 
             # Technically required, but not found must not happen
-            assert finding != "not-found"
+            assert finding != "not-found", early_module_name
 
             self.standard_library_modules.add(
                 makeModuleUsageAttempt(
@@ -1062,3 +1071,19 @@ class PythonExtensionModule(PythonModuleBase):
 
     def getParentModule(self):
         return self
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
