@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Assignment related nodes.
 
 The most simple assignment statement ``a = b`` is what we have here. All others
@@ -179,20 +165,7 @@ class StatementAssignmentVariableMixin(object):
         # is such a code for the type shape.
         return True
 
-    def _transferState(self, result):
-        self.variable_trace.assign_node = result
-        result.variable_trace = self.variable_trace
-        self.variable_trace = None
-        result.parent = self.parent
-
-    def _considerSpecialization(self, old_source):
-        # Specialize if possible, might have become that way only recently.
-        # return driven, pylint: disable=too-many-return-statements
-        source = self.subnode_source
-
-        if source is old_source:
-            return self, None, None
-
+    def _considerSpecialization(self, old_source, source):
         if source.isCompileTimeConstant():
             result = makeStatementAssignmentVariableConstant(
                 source=source,
@@ -202,8 +175,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -219,8 +190,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -235,8 +204,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -251,8 +218,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -267,8 +232,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -285,6 +248,10 @@ class StatementAssignmentVariableMixin(object):
     def hasVeryTrustedValue(self):
         """Does this assignment node have a very trusted value."""
 
+    @abstractmethod
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        """Set the value trace for the assignment form."""
+
 
 class StatementAssignmentVariableGeneric(
     StatementAssignmentVariableMixin, StatementAssignmentVariableGenericBase
@@ -404,17 +371,26 @@ Removed assignment of %s from itself which is known to be defined."""
                     % variable.getDescription(),
                 )
 
+        if source is old_source:
+            result = self, None, None
+        else:
+            result = self._considerSpecialization(old_source, source)
+            result[0].parent = self.parent
+
+        result[0].computeStatementAssignmentTraceUpdate(trace_collection, source)
+
+        return result
+
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
         self.variable_trace = trace_collection.onVariableSet(
-            variable=variable, version=self.variable_version, assign_node=self
+            variable=self.variable, version=self.variable_version, assign_node=self
         )
 
         # TODO: Determine from future use of assigned variable, if this is needed at all.
         trace_collection.removeKnowledge(source)
 
-        return self._considerSpecialization(old_source)
-
     def hasVeryTrustedValue(self):
         """Does this assignment node have a very trusted value."""
         return self.subnode_source.hasVeryTrustedValue()
@@ -594,12 +570,18 @@ Assignment raises exception in assigned value, removed assignment.""",
 
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSet(
             variable=variable, version=self.variable_version, assign_node=self
         )
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSet(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -642,6 +624,7 @@ class StatementAssignmentVariableConstantMutable(
 
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSet(
             variable=variable, version=self.variable_version, assign_node=self
         )
@@ -682,6 +665,11 @@ class StatementAssignmentVariableConstantMutable(
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSet(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -785,12 +773,18 @@ class StatementAssignmentVariableConstantImmutable(
                         % self.getVariableName(),
                     )
 
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSetToUnescapableValue(
             variable=variable, version=self.variable_version, assign_node=self
         )
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSetToUnescapableValue(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -868,12 +862,18 @@ class StatementAssignmentVariableHardValue(
 Assignment raises exception in assigned value, removed assignment.""",
             )
 
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSetToVeryTrustedValue(
             variable=variable, version=self.variable_version, assign_node=self
         )
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSetToVeryTrustedValue(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -916,7 +916,9 @@ class StatementAssignmentVariableFromVariable(
 
         # Assigning from and to the same variable, can be optimized away
         # immediately, there is no point in doing it. Exceptions are of course
-        # module variables that collide with built-in names.
+        # module variables that collide with built-in names. TODO: In
+        # specialization this could be considered right away as its own node
+        # type, waiting for it to compute like this.
         if not variable.isModuleVariable() and old_source.getVariable() is variable:
             # A variable access that has a side effect, must be preserved,
             # so it can e.g. raise an exception, otherwise we can be fully
@@ -947,25 +949,48 @@ Removed assignment of %s from itself which is known to be defined."""
         # Let assignment source may re-compute first.
         source = trace_collection.onExpression(self.subnode_source)
 
+        if source is old_source:
+            result = self, None, None
+        else:
+            if source.willRaiseAnyException():
+                result = makeStatementExpressionOnlyReplacementNode(
+                    expression=source, node=self
+                )
+
+                return (
+                    result,
+                    "new_raise",
+                    """\
+Assignment raises exception in assigned variable access, removed assignment.""",
+                )
+
+            result = self._considerSpecialization(old_source, source)
+            result[0].parent = self.parent
+
+        result[0].computeStatementAssignmentTraceUpdate(trace_collection, source)
+
+        return result
+
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
         if source.isExpressionVariableRef():
             self.variable_trace = trace_collection.onVariableSetAliasing(
-                variable=variable,
+                variable=self.variable,
                 version=self.variable_version,
                 assign_node=self,
                 source=source,
             )
         else:
             # Set-up the trace to the trace collection, so future references will
-            # find this assignment.
+            # find this assignment. TODO: We should for non-variables make sure we do
+            # always specialize, since this is no longer a variable once it was
+            # resolved.
             self.variable_trace = trace_collection.onVariableSet(
-                variable=variable, version=self.variable_version, assign_node=self
+                variable=self.variable, version=self.variable_version, assign_node=self
             )
 
             # TODO: Determine from future use of assigned variable, if this is needed at all.
             trace_collection.removeKnowledge(source)
 
-        return self._considerSpecialization(old_source)
-
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -1021,17 +1046,26 @@ Removed assignment of %s from itself which is known to be defined."""
         # Let assignment source may re-compute first.
         source = trace_collection.onExpression(self.subnode_source)
 
+        if source is old_source:
+            result = self, None, None
+        else:
+            result = self._considerSpecialization(old_source, source)
+            result[0].parent = self.parent
+
+        result[0].computeStatementAssignmentTraceUpdate(trace_collection, source)
+
+        return result
+
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
         self.variable_trace = trace_collection.onVariableSet(
-            variable=variable, version=self.variable_version, assign_node=self
+            variable=self.variable, version=self.variable_version, assign_node=self
         )
 
         # TODO: Determine from future use of assigned variable, if this is needed at all.
         trace_collection.removeKnowledge(source)
 
-        return self._considerSpecialization(old_source)
-
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -1124,3 +1158,19 @@ def makeStatementAssignmentVariable(
             variable_version=variable_version,
             source_ref=source_ref,
         )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
