@@ -1,25 +1,12 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Tools for tracing memory usage at compiled time.
 
 """
 
-from nuitka.Tracing import printLine
+from nuitka.containers.OrderedDicts import OrderedDict
+from nuitka.Tracing import memory_logger, printLine
 
 from .Utils import isMacOS, isWin32Windows
 
@@ -49,11 +36,11 @@ def getOwnProcessMemoryUsage():
             ]
 
         GetProcessMemoryInfo = ctypes.windll.psapi.GetProcessMemoryInfo
-        GetProcessMemoryInfo.argtypes = [
+        GetProcessMemoryInfo.argtypes = (
             ctypes.wintypes.HANDLE,
             ctypes.POINTER(PROCESS_MEMORY_COUNTERS_EX),
             ctypes.wintypes.DWORD,
-        ]
+        )
         GetProcessMemoryInfo.restype = ctypes.wintypes.BOOL
 
         counters = PROCESS_MEMORY_COUNTERS_EX()
@@ -80,10 +67,42 @@ def getOwnProcessMemoryUsage():
         return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss * factor
 
 
-def getHumanReadableProcessMemoryUsage(value=None):
-    if value is None:
-        value = getOwnProcessMemoryUsage()
+def getHumanReadableProcessMemoryUsage():
+    return formatMemoryUsageValue(getOwnProcessMemoryUsage())
 
+
+class MemoryWatch(object):
+    def __init__(self):
+        self.start = getOwnProcessMemoryUsage()
+        self.stop = None
+
+    def finish(self, message):
+        self.stop = getOwnProcessMemoryUsage()
+
+        _logMemoryInfo(message, self.value())
+
+    def asStr(self):
+        return formatMemoryUsageValue(self.value())
+
+    def value(self):
+        return self.stop - self.start
+
+
+_memory_infos = OrderedDict()
+
+
+def getMemoryInfos():
+    return _memory_infos
+
+
+def collectMemoryUsageValue(memory_usage_name):
+    assert memory_usage_name not in _memory_infos, memory_usage_name
+    _memory_infos[memory_usage_name] = getOwnProcessMemoryUsage()
+
+    return _memory_infos[memory_usage_name]
+
+
+def formatMemoryUsageValue(value):
     if abs(value) < 1024 * 1014:
         return "%.2f KB (%d bytes)" % (value / 1024.0, value)
     elif abs(value) < 1024 * 1014 * 1024:
@@ -94,16 +113,15 @@ def getHumanReadableProcessMemoryUsage(value=None):
         return "%d bytes" % value
 
 
-class MemoryWatch(object):
-    def __init__(self):
-        self.start = getOwnProcessMemoryUsage()
-        self.stop = None
+def _logMemoryInfo(message, memory_usage):
+    if message:
+        memory_logger.info("%s: %s" % (message, formatMemoryUsageValue(memory_usage)))
 
-    def finish(self):
-        self.stop = getOwnProcessMemoryUsage()
 
-    def asStr(self):
-        return getHumanReadableProcessMemoryUsage(self.stop - self.start)
+def reportMemoryUsage(identifier, message):
+    memory_usage = collectMemoryUsageValue(identifier)
+
+    _logMemoryInfo(message, memory_usage)
 
 
 def startMemoryTracing():
@@ -129,3 +147,19 @@ def showMemoryTrace():
             if count == 50:
                 break
             printLine(stat)
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
